package jalview.io;

import org.apache.axis.client.*;
import javax.xml.namespace.QName;
import java.util.*;
import jalview.datamodel.*;
import jalview.gui.*;
import javax.swing.*;

public class WSWUBlastClient
{
  CutAndPasteTransfer cap = new CutAndPasteTransfer(false);

  public WSWUBlastClient(AlignmentI al, ArrayList ids)
  {
    JInternalFrame frame = new JInternalFrame();
    cap.formatForOutput();
    frame.setContentPane(cap);
    cap.setText("To display sequence features an exact Uniprot id with 100% sequence identity match must be entered."
                +"\nIn order to display these features, try changing the names of your sequences to the ids suggested below.");
    Desktop.addInternalFrame(frame, "BLASTing for unidentified sequences ", 800,300);

    for(int i=0; i<ids.size(); i++)
    {
     SequenceI sequence =  al.findName( ids.get(i).toString() );
     StringBuffer nonGapped = new StringBuffer();
      for (int n = 0; n < sequence.getSequence().length(); n++)
      {
        if (!jalview.util.Comparison.isGap(sequence.getCharAt(n)))
          nonGapped.append(sequence.getCharAt(n));
      }

     BlastThread thread = new BlastThread(ids.get(i).toString(),  nonGapped.toString());
     thread.start();
     }

  }

  class BlastThread extends Thread
  {
    String sequence;
    String id;
    BlastThread(String id, String sequence)
    {
      this.sequence = sequence;
      this.id = id;
    }

    public void run()
    {
      FindUniprotIDFromUnknownSequence(id, sequence);
    }
  }

  public void FindUniprotIDFromUnknownSequence (String id, String s)
  {
        HashMap params = new HashMap();
        params.put("database", "uniprot");
        params.put("sensitivity","low");
        params.put("sort","totalscore");
        params.put("matrix","pam10");
        params.put("program","blastp");
        params.put("alignments","5");
        params.put("outformat","xml");
        byte[] sequence = s.getBytes();

        try {
            Call call = (Call) new Service().createCall();
            call.setTargetEndpointAddress (new java.net.URL("http://www.ebi.ac.uk/cgi-bin/webservices/WSWUBlast"));
            call.setOperationName(new QName("WSWUBlast", "doWUBlast"));

            String result = (String) call.invoke(new Object[] {params,sequence});
            parseResult(id, result);
            }
        catch (Exception exp) {
             System.err.println ("ERROR:\n" + exp.toString());
            exp.printStackTrace();
        }
    }

    void parseResult(String id1, String res)
    {
      StringTokenizer st = new StringTokenizer(res, "\n");
      String data;
      String id2;
      int maxFound = 90;
      StringBuffer buffer = new StringBuffer("\n\n"+id1+" :");

      while( st.hasMoreTokens() )
      {
        data = st.nextToken();

        if(data.indexOf("database=\"uniprot\" id=")>-1)
        {
          int index =  data.indexOf("database=\"uniprot\" id=")+ 23;
          id2 = data.substring( index, data.indexOf("\"", index) );
          while( data.indexOf("</alignment>")==-1)
          {
            data = st.nextToken();
            if(data.indexOf("<identity>")>-1)
            {
              int value = Integer.parseInt( data.substring(data.indexOf("<identity>")+10, data.indexOf("</identity>")));
              if(value>=maxFound)
              {
                maxFound = value;
                buffer.append(" "+ id2 + " " + value+"%; ");
              }
            }
          }

        }


      }

      cap.setText(cap.getText()+buffer.toString());
    }

}
